'MonoLcdDriver3 Test
'Test of the Assembly parallel interface driver 120x32 Mono LCD (with icons)
'Copyright (C) 2011 Raymond Allen, Rayslgoic.com, LLC  
'Ver1Rev0:  Initial Version
'
'This version does 15 columns by 2 rows of characters with scaled down version of ROM font
'Graphics tiles show every other pixel
'
'This test does a modified version of the Graphics Demo for TV
'It also shows the loop count and also minutes and seconds in the icon area
'Also demonstrates how to set regular and special icons
'
'Default configuration of driver is for connecting to upper pins of Prop Platform
'A simple change in the driver allows you to use the lower pins, see the notes in the driver
'You can also print very large characters on the screen using str(),dec(), etc. instead of the graphics
'
'The LCD driver doubles as a seconds clock
'You can use it by itself to simply count seconds
'Or, you can add in PropTimeMono.spin for advanced time operations

CON

  _clkmode = xtal1 + pll16x
  _xinfreq =5_000_000

CON
  'Double buffering is not used here because LCD's GRAM serves as the second buffer
  _stack = (x_tiles*y_tiles*16*4 + 128) >> 2   'accomodate display memory and stack
  x_tiles = 15  'only 7.5 in reality (120 columns)
  y_tiles = 4  '32 rows

  'Since we're not double-buffering, pointing both these display buffer pointers to the same place
  'Shifting the bitmap_base pointer to the end of memory, where display_base also points 
  bitmap_base = ($8000)-x_tiles*y_tiles*16*4-128
  display_base = bitmap_base


CON
  'Set screen for 120x32 pixels
  cols = 15
  rows = 4
  screensize = cols * rows
  lastrow = screensize - cols

  lines = 5
  thickness = 2
    
VAR
  'screen variables  
  long  col, row, color, flag    

  'graphics demo vars
  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]
  'can use mouse
  long  mousex, mousey
  'screen and color arrays
  word  screen[x_tiles * y_tiles]
  long  colors[64]    'We're only using 5
  
OBJ
  lcd   : "MonoLcdDriver3" 'Driver being tested
  gr    : "graphics"       'Parallax provided graphics driver
  mouse : "mouse"         'This driver isn't started in this example
  time  : "PropTimeMono"  'This is optional, but the driver keeps time, so add this for advanced time operations



PUB start(mode) : okay|c,p,i
  'Start the LCD driver
  'The calling cog will control the reset pin
  'All other interface pins controlled by assembly driver cog
   lcd.Start(@screen,@colors)

   Colors[0]:=$FF00FF00   'Black on white text
   colors[1]:=$FFFF0000   '
   Colors[2]:=$00FF00FF   'White on black text
   colors[3]:=$0000FFFF   '
   colors[4]:=$FFFFFF00  'Graphics colors

  'If using the time object, need to initialize it
  'Calculate current time
  Time.SetLocalTime(2, 17, 2011, 4, 29, 00)  '(month, day, year, hours, minutes,seconds)
  'Set the clock in the lcd driver
  LCD.SetPropTime(Time.Time)   'sends the 32-bit time to the lcd driver

  'Set a few icons
  'Note that 4 icons are special (fast to change) (IconPower = 0, IconAntenna = 1, IconSpeaker = 2, IconMute = 3)
  lcd.SetSpecialIcon(lcd#IconPower,true)

  'The other icons take a couple nanoseconds longer to set..
  lcd.SetIcon(lcd#IconPlay,true)

  out(0)

  str(string("15x2 Text usingscaled ROM font"))
   
  lcd.UpdateScreen  'Copy buffer to LCD 
  waitcnt(cnt+clkfreq*10) 'wait 10 seconds
  
  'Start Graphics Demo (never ending loop)  
  GraphicsDemo  
    

PUB GraphicsDemo|i,j,k,dx,dy,pq,pp, rr, numx, numchr,kk   
   'init colors
  'repeat i from 0 to 63
  '  colors[i] := $00001010 * (i*12/y_tiles+4) & $F + $2B060C02     'slight change here to match

  'init tile screen
  repeat dx from 0 to cols-1'tv_hc - 1
    repeat dy from 0 to rows-1'tv_vc - 1
      screen[dy * cols + dx] := display_base >> 6 + dy + dx * rows + 4<<10'((dy & $3F) << 10)


  
  'init bouncing lines
  i := 1001
  j := 123123
  k := 8776434
  repeat i from 0 to lines - 1
    x[i] := ?j // 64
    y[i] := k? // 48
    repeat until xs[i] := k? ~> 29
    repeat until ys[i] := ?j ~> 29

  'start and setup graphics
  gr.start
  'gr.setup(16, 12, 128, 96, bitmap_base)        'modified for the variable size
  gr.setup(x_tiles, y_tiles, x_tiles*16/2, y_tiles*16/2, bitmap_base)

  'start mouse
  'Pins 24 and 25 are used by the PSM LCD, so we dont' want to start the mouse with those pins!
  'If you add a mouse with free pins (like P30..P31), then un-comment and update the next line
  'mouse.start(24, 25)

  k:=0
  'main loop
  repeat
    'first, clear bitmap
    gr.clear

    'draw spinning triangles
    gr.colorwidth(3,0)
    repeat i from 1 to 8
      gr.vec(0, 0, (k & $7F) << 3 + i << 5, k << 6 + i << 8, @vecdef)





    'draw expanding mouse crosshairs
    gr.colorwidth(2,k>>2)
    mousex := mousex + mouse.delta_x #> -128 <# 127
    mousey := mousey + mouse.delta_y #> -96 <# 95
    gr.pix(mousex, mousey, k>>4 & $7, @pixdef)
    
    'if left mouse button pressed, throw snowballs
    if mouse.button(0)
      gr.width(pq & $F)
      gr.color(2)
      pp := (pq & $F)*(pq & $F) + 5
      pq++
      gr.arc(mousex, mousey, pp, pp>>1, -k * 200, $200, 8, 0)
    else
      pq~

    'if right mouse button pressed, pause
    repeat while mouse.button(1)

    'draw expanding pixel halo
    gr.colorwidth(1,k)
    gr.arc(0,0,80,30,-k<<5,$2000/9,9,0)

    'step bouncing lines
    repeat i from 0 to lines - 1
      if ||~x[i] > 60
        -xs[i]
      if ||~y[i] > 40
        -ys[i]
      x[i] += xs[i]
      y[i] += ys[i]

    'draw bouncing lines
    gr.colorwidth(1,thickness)
    gr.plot(~x[0], ~y[0])
    repeat i from 1 to lines - 1
      gr.line(~x[i],~y[i])
    gr.line(~x[0], ~y[0])

    'draw spinning stars and revolving crosshairs and dogs
    gr.colorwidth(2,0)
    repeat i from 0 to 7
      gr.vecarc(80,50,30,30,-(i<<10+k<<6),$40,-(k<<7),@vecdef2)
      gr.pixarc(-80,-40,30,30,i<<10+k<<6,0,@pixdef2)
      gr.pixarc(-80,-40,20,20,-(i<<10+k<<6),0,@pixdef)

    'draw small box with text
    gr.colorwidth(1,14)
    gr.box(60,-80,60,16)
    gr.textmode(2,2,6,5)
    gr.colorwidth(2,2)
    gr.text(50,-20,@pchip)

    'draw incrementing digit
    if not ++numx & 7
      numchr++
    if numchr < "0" or numchr > "9"
      numchr := "0"
    gr.textmode(3,3,6,5)
    gr.colorwidth(1,3)
    gr.text(-100,15,@numchr)

    'waitcnt(cnt+clkfreq/80)  'Need a brief delay here to let graphics complete last operation...
             'Don't need above delay when showing time (because it takes a while)
    'Show counter in icon number area
    lcd.SetIconNumber(k)

    ShowTime  'update the minutes and seconds
    
    lcd.UpdateScreen  'Copy buffer to LCD 

    'increment counter that makes everything change
    k++

    if k==30
      'repeat



       


PUB ShowTime|i,pString 'Show current time (minutes and seconds) in icon area
  i:=LCD.GetPropTime
  time.SetUnixTime(i)
  pString:=time.StrTime(1)  'Format time into a string  
  LCD.SetIconTime(byte[pString+3]-"0",byte[pString+4]-"0",byte[pString+6]-"0",byte[pString+7]-"0")




DAT


vecdef                  word    $4000+$2000/3*0         'triangle
                        word    50
                        word    $8000+$2000/3*1+1
                        word    50
                        word    $8000+$2000/3*2-1
                        word    50
                        word    $8000+$2000/3*0
                        word    50
                        word    0

vecdef2                 word    $4000+$2000/12*0        'star
                        word    50
                        word    $8000+$2000/12*1
                        word    20
                        word    $8000+$2000/12*2
                        word    50
                        word    $8000+$2000/12*3
                        word    20
                        word    $8000+$2000/12*4
                        word    50
                        word    $8000+$2000/12*5
                        word    20
                        word    $8000+$2000/12*6
                        word    50
                        word    $8000+$2000/12*7
                        word    20
                        word    $8000+$2000/12*8
                        word    50
                        word    $8000+$2000/12*9
                        word    20
                        word    $8000+$2000/12*10
                        word    50
                        word    $8000+$2000/12*11
                        word    20
                        word    $8000+$2000/12*0
                        word    50
                        word    0

pixdef                  word                            'crosshair
                        byte    2,7,3,3
                        word    %%00333000,%%00000000
                        word    %%03020300,%%00000000
                        word    %%30020030,%%00000000
                        word    %%32222230,%%00000000
                        word    %%30020030,%%02000000
                        word    %%03020300,%%22200000
                        word    %%00333000,%%02000000

pixdef2                 word                            'dog
                        byte    1,4,0,3
                        word    %%20000022
                        word    %%02222222
                        word    %%02222200
                        word    %%02000200

pchip                   byte    "Propeller",0           'text  





PUB str(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    out(byte[stringptr++])


PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10


PUB hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB bin(value, digits)

'' Print a binary number

  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")


PUB out(c) | i, k

'' Output a character
''
''     $00 = clear screen
''     $01 = home
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = set X position (X follows)
''     $0B = set Y position (Y follows)
''     $0C = set color (color follows)
''     $0D = return
''  others = printable characters

  case flag
    $00: case c
           $00: wordfill(@screen, $220, screensize)
                col := row := 0
           $01: col := row := 0
           $08: if col
                  col--
           $09: repeat
                  print(" ")
                while col & 7
           $0A..$0C: flag := c
                     return
           $0D: 'newline   'row must stay 0  
           other: print(c)
    $0A: col := c // cols
    $0B: row := c // rows  
    $0C: color := c & 7
  flag := 0
  
PRI print(c)|k,i
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols
        col:=0
        row:=2



PRI DelayMS(ms)   'milliseconds delay
  waitcnt(cnt+ms*(clkfreq/1000))


 